@@ -19,10 +19,10 @@
 #include "src/cpp/ext/csm/metadata_exchange.h"
 
 #include "absl/functional/any_invocable.h"
-#include "api/include/opentelemetry/metrics/provider.h"
 #include "gmock/gmock.h"
 #include "google/cloud/opentelemetry/resource_detector.h"
 #include "gtest/gtest.h"
+#include "opentelemetry/metrics/provider.h"
 #include "opentelemetry/sdk/metrics/meter_provider.h"
 #include "opentelemetry/sdk/metrics/metric_reader.h"
 
@@ -35,7 +35,7 @@
 #include "src/core/lib/gprpp/env.h"
 #include "src/cpp/ext/csm/csm_observability.h"
 #include "src/cpp/ext/otel/otel_plugin.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 #include "test/cpp/end2end/test_service_impl.h"
 #include "test/cpp/ext/otel/otel_test_library.h"
 
@@ -43,6 +43,37 @@ namespace grpc {
 namespace testing {
 namespace {
 
+using OptionalLabelKey =
+    grpc_core::ClientCallTracer::CallAttemptTracer::OptionalLabelKey;
+using ::testing::ElementsAre;
+using ::testing::Pair;
+
+opentelemetry::sdk::resource::Resource TestGkeResource() {
+  opentelemetry::sdk::common::AttributeMap attributes;
+  attributes.SetAttribute("cloud.platform", "gcp_kubernetes_engine");
+  attributes.SetAttribute("k8s.pod.name", "pod");
+  attributes.SetAttribute("k8s.container.name", "container");
+  attributes.SetAttribute("k8s.namespace.name", "namespace");
+  attributes.SetAttribute("k8s.cluster.name", "cluster");
+  attributes.SetAttribute("cloud.region", "region");
+  attributes.SetAttribute("cloud.account.id", "id");
+  return opentelemetry::sdk::resource::Resource::Create(attributes);
+}
+
+opentelemetry::sdk::resource::Resource TestGceResource() {
+  opentelemetry::sdk::common::AttributeMap attributes;
+  attributes.SetAttribute("cloud.platform", "gcp_compute_engine");
+  attributes.SetAttribute("cloud.availability_zone", "zone");
+  attributes.SetAttribute("cloud.account.id", "id");
+  return opentelemetry::sdk::resource::Resource::Create(attributes);
+}
+
+opentelemetry::sdk::resource::Resource TestUnknownResource() {
+  opentelemetry::sdk::common::AttributeMap attributes;
+  attributes.SetAttribute("cloud.platform", "random");
+  return opentelemetry::sdk::resource::Resource::Create(attributes);
+}
+
 class TestScenario {
  public:
   enum class ResourceType : std::uint8_t { kGke, kGce, kUnknown };
@@ -91,32 +122,6 @@ class TestScenario {
   XdsBootstrapSource bootstrap_source() const { return bootstrap_source_; }
 
  private:
-  static opentelemetry::sdk::resource::Resource TestGkeResource() {
-    opentelemetry::sdk::common::AttributeMap attributes;
-    attributes.SetAttribute("cloud.platform", "gcp_kubernetes_engine");
-    attributes.SetAttribute("k8s.pod.name", "pod");
-    attributes.SetAttribute("k8s.container.name", "container");
-    attributes.SetAttribute("k8s.namespace.name", "namespace");
-    attributes.SetAttribute("k8s.cluster.name", "cluster");
-    attributes.SetAttribute("cloud.region", "region");
-    attributes.SetAttribute("cloud.account.id", "id");
-    return opentelemetry::sdk::resource::Resource::Create(attributes);
-  }
-
-  static opentelemetry::sdk::resource::Resource TestGceResource() {
-    opentelemetry::sdk::common::AttributeMap attributes;
-    attributes.SetAttribute("cloud.platform", "gcp_compute_engine");
-    attributes.SetAttribute("cloud.availability_zone", "zone");
-    attributes.SetAttribute("cloud.account.id", "id");
-    return opentelemetry::sdk::resource::Resource::Create(attributes);
-  }
-
-  static opentelemetry::sdk::resource::Resource TestUnknownResource() {
-    opentelemetry::sdk::common::AttributeMap attributes;
-    attributes.SetAttribute("cloud.platform", "random");
-    return opentelemetry::sdk::resource::Resource::Create(attributes);
-  }
-
   ResourceType type_;
   XdsBootstrapSource bootstrap_source_;
 };
@@ -152,9 +157,7 @@ class MetadataExchangeTest
     : public OpenTelemetryPluginEnd2EndTest,
       public ::testing::WithParamInterface<TestScenario> {
  protected:
-  void Init(absl::flat_hash_set<absl::string_view> metric_names,
-            bool enable_client_side_injector = true,
-            std::map<std::string, std::string> labels_to_inject = {}) {
+  void Init(Options options, bool enable_client_side_injector = true) {
     const char* kBootstrap =
         "{\"node\": {\"id\": "
         "\"projects/1234567890/networks/mesh:mesh-id/nodes/"
@@ -163,7 +166,7 @@ class MetadataExchangeTest
       case TestScenario::XdsBootstrapSource::kFromFile: {
         ASSERT_EQ(bootstrap_file_name_, nullptr);
         FILE* bootstrap_file =
-            gpr_tmpfile("gcp_observability_config", &bootstrap_file_name_);
+            gpr_tmpfile("xds_bootstrap", &bootstrap_file_name_);
         fputs(kBootstrap, bootstrap_file);
         fclose(bootstrap_file);
         grpc_core::SetEnv("GRPC_XDS_BOOTSTRAP", bootstrap_file_name_);
@@ -174,19 +177,18 @@ class MetadataExchangeTest
         break;
     }
     OpenTelemetryPluginEnd2EndTest::Init(std::move(
-        Options()
-            .set_metric_names(std::move(metric_names))
+        options
             .add_plugin_option(std::make_unique<MeshLabelsPluginOption>(
                 GetParam().GetTestResource().GetAttributes()))
-            .set_labels_to_inject(std::move(labels_to_inject))
-            .set_target_selector(
-                [enable_client_side_injector](absl::string_view /*target*/) {
+            .set_channel_scope_filter(
+                [enable_client_side_injector](
+                    const OpenTelemetryPluginBuilder::ChannelScope& /*scope*/) {
                   return enable_client_side_injector;
                 })));
   }
 
   ~MetadataExchangeTest() override {
-    grpc_core::UnsetEnv("GRPC_GCP_OBSERVABILITY_CONFIG");
+    grpc_core::UnsetEnv("GRPC_XDS_BOOTSTRAP_CONFIG");
     grpc_core::UnsetEnv("GRPC_XDS_BOOTSTRAP");
     if (bootstrap_file_name_ != nullptr) {
       remove(bootstrap_file_name_);
@@ -275,8 +277,9 @@ class MetadataExchangeTest
 
 // Verify that grpc.client.attempt.started does not get service mesh attributes
 TEST_P(MetadataExchangeTest, ClientAttemptStarted) {
-  Init(/*metric_names=*/{
-      grpc::OpenTelemetryPluginBuilder::kClientAttemptStartedInstrumentName});
+  Init(std::move(
+      Options().set_metric_names({grpc::OpenTelemetryPluginBuilder::
+                                      kClientAttemptStartedInstrumentName})));
   SendRPC();
   const char* kMetricName = "grpc.client.attempt.started";
   auto data = ReadCurrentMetricsData(
@@ -299,8 +302,9 @@ TEST_P(MetadataExchangeTest, ClientAttemptStarted) {
 }
 
 TEST_P(MetadataExchangeTest, ClientAttemptDuration) {
-  Init(/*metric_names=*/{
-      grpc::OpenTelemetryPluginBuilder::kClientAttemptDurationInstrumentName});
+  Init(std::move(
+      Options().set_metric_names({grpc::OpenTelemetryPluginBuilder::
+                                      kClientAttemptDurationInstrumentName})));
   SendRPC();
   const char* kMetricName = "grpc.client.attempt.duration";
   auto data = ReadCurrentMetricsData(
@@ -324,9 +328,8 @@ TEST_P(MetadataExchangeTest, ClientAttemptDuration) {
 
 // Verify that grpc.server.call.started does not get service mesh attributes
 TEST_P(MetadataExchangeTest, ServerCallStarted) {
-  Init(
-      /*metric_names=*/{
-          grpc::OpenTelemetryPluginBuilder::kServerCallStartedInstrumentName});
+  Init(std::move(Options().set_metric_names(
+      {grpc::OpenTelemetryPluginBuilder::kServerCallStartedInstrumentName})));
   SendRPC();
   const char* kMetricName = "grpc.server.call.started";
   auto data = ReadCurrentMetricsData(
@@ -345,9 +348,8 @@ TEST_P(MetadataExchangeTest, ServerCallStarted) {
 }
 
 TEST_P(MetadataExchangeTest, ServerCallDuration) {
-  Init(
-      /*metric_names=*/{
-          grpc::OpenTelemetryPluginBuilder::kServerCallDurationInstrumentName});
+  Init(std::move(Options().set_metric_names(
+      {grpc::OpenTelemetryPluginBuilder::kServerCallDurationInstrumentName})));
   SendRPC();
   const char* kMetricName = "grpc.server.call.duration";
   auto data = ReadCurrentMetricsData(
@@ -369,10 +371,10 @@ TEST_P(MetadataExchangeTest, ServerCallDuration) {
 
 // Test that the server records unknown when the client does not send metadata
 TEST_P(MetadataExchangeTest, ClientDoesNotSendMetadata) {
-  Init(
-      /*metric_names=*/{grpc::OpenTelemetryPluginBuilder::
-                            kServerCallDurationInstrumentName},
-      /*enable_client_side_injector=*/false);
+  Init(std::move(
+           Options().set_metric_names({grpc::OpenTelemetryPluginBuilder::
+                                           kServerCallDurationInstrumentName})),
+       /*enable_client_side_injector=*/false);
   SendRPC();
   const char* kMetricName = "grpc.server.call.duration";
   auto data = ReadCurrentMetricsData(
@@ -398,11 +400,15 @@ TEST_P(MetadataExchangeTest, ClientDoesNotSendMetadata) {
 }
 
 TEST_P(MetadataExchangeTest, VerifyCsmServiceLabels) {
-  Init(/*metric_names=*/{grpc::OpenTelemetryPluginBuilder::
-                             kClientAttemptDurationInstrumentName},
-       /*enable_client_side_injector=*/true,
-       // Injects CSM service labels to be recorded in the call.
-       {{"service_name", "myservice"}, {"service_namespace", "mynamespace"}});
+  Init(std::move(
+      Options()
+          .set_metric_names({grpc::OpenTelemetryPluginBuilder::
+                                 kClientAttemptDurationInstrumentName})
+          .set_labels_to_inject(
+              {{OptionalLabelKey::kXdsServiceName,
+                grpc_core::RefCountedStringValue("myservice")},
+               {OptionalLabelKey::kXdsServiceNamespace,
+                grpc_core::RefCountedStringValue("mynamespace")}})));
   SendRPC();
   const char* kMetricName = "grpc.client.attempt.duration";
   auto data = ReadCurrentMetricsData(
@@ -418,6 +424,179 @@ TEST_P(MetadataExchangeTest, VerifyCsmServiceLabels) {
             "mynamespace");
 }
 
+TEST_P(MetadataExchangeTest, Retries) {
+  Init(std::move(
+      Options()
+          .set_metric_names({grpc::OpenTelemetryPluginBuilder::
+                                 kClientAttemptDurationInstrumentName})
+          .set_service_config(
+              "{\n"
+              "  \"methodConfig\": [ {\n"
+              "    \"name\": [\n"
+              "      { \"service\": \"grpc.testing.EchoTestService\" }\n"
+              "    ],\n"
+              "    \"retryPolicy\": {\n"
+              "      \"maxAttempts\": 3,\n"
+              "      \"initialBackoff\": \"0.1s\",\n"
+              "      \"maxBackoff\": \"120s\",\n"
+              "      \"backoffMultiplier\": 1,\n"
+              "      \"retryableStatusCodes\": [ \"ABORTED\" ]\n"
+              "    }\n"
+              "  } ]\n"
+              "}")));
+  EchoRequest request;
+  request.mutable_param()->mutable_expected_error()->set_code(
+      StatusCode::ABORTED);
+  EchoResponse response;
+  grpc::ClientContext context;
+  grpc::Status status = stub_->Echo(&context, request, &response);
+  const char* kMetricName = "grpc.client.attempt.duration";
+  auto data = ReadCurrentMetricsData(
+      [&](const absl::flat_hash_map<
+          std::string,
+          std::vector<opentelemetry::sdk::metrics::PointDataAttributes>>&
+              data) {
+        return !data.contains(kMetricName) ||
+               absl::get<opentelemetry::sdk::metrics::HistogramPointData>(
+                   data.at(kMetricName)[0].point_data)
+                       .count_ != 3;
+      });
+  ASSERT_EQ(absl::get<opentelemetry::sdk::metrics::HistogramPointData>(
+                data.at(kMetricName)[0].point_data)
+                .count_,
+            3);
+  VerifyServiceMeshAttributes(data.at(kMetricName)[0].attributes,
+                              /*is_client=*/true);
+}
+
+// Creates a serialized slice with labels for metadata exchange based on \a
+// resource.
+grpc_core::Slice RemoteMetadataSliceFromResource(
+    const opentelemetry::sdk::resource::Resource& resource) {
+  return grpc::internal::ServiceMeshLabelsInjector(resource.GetAttributes())
+      .TestOnlySerializedLabels()
+      .Ref();
+}
+
+std::vector<std::pair<absl::string_view, absl::string_view>> LabelsFromIterable(
+    grpc::internal::MeshLabelsIterable* iterable) {
+  std::vector<std::pair<absl::string_view, absl::string_view>> labels;
+  while (true) {
+    auto label = iterable->Next();
+    if (!label.has_value()) break;
+    labels.push_back(*std::move(label));
+  }
+  EXPECT_EQ(labels.size(), iterable->Size());
+  return labels;
+}
+
+std::string PrettyPrintLabels(
+    const std::vector<std::pair<absl::string_view, absl::string_view>>&
+        labels) {
+  std::vector<std::string> strings;
+  strings.reserve(labels.size());
+  for (const auto& pair : labels) {
+    strings.push_back(
+        absl::StrFormat("{\"%s\" : \"%s\"}", pair.first, pair.second));
+  }
+  return absl::StrJoin(strings, ", ");
+}
+
+TEST(MeshLabelsIterableTest, NoRemoteMetadata) {
+  std::vector<std::pair<absl::string_view, std::string>> local_labels = {
+      {"csm.workload_canonical_service", "canonical_service"},
+      {"csm.mesh_id", "mesh"}};
+  grpc::internal::MeshLabelsIterable iterable(local_labels, grpc_core::Slice());
+  auto labels = LabelsFromIterable(&iterable);
+  EXPECT_FALSE(iterable.GotRemoteLabels());
+  EXPECT_THAT(
+      labels,
+      ElementsAre(Pair("csm.workload_canonical_service", "canonical_service"),
+                  Pair("csm.mesh_id", "mesh"),
+                  Pair("csm.remote_workload_type", "unknown"),
+                  Pair("csm.remote_workload_canonical_service", "unknown")))
+      << PrettyPrintLabels(labels);
+}
+
+TEST(MeshLabelsIterableTest, RemoteGceTypeMetadata) {
+  std::vector<std::pair<absl::string_view, std::string>> local_labels = {
+      {"csm.workload_canonical_service", "canonical_service"},
+      {"csm.mesh_id", "mesh"}};
+  grpc::internal::MeshLabelsIterable iterable(
+      local_labels, RemoteMetadataSliceFromResource(TestGceResource()));
+  auto labels = LabelsFromIterable(&iterable);
+  EXPECT_TRUE(iterable.GotRemoteLabels());
+  EXPECT_THAT(
+      labels,
+      ElementsAre(
+          Pair("csm.workload_canonical_service", "canonical_service"),
+          Pair("csm.mesh_id", "mesh"),
+          Pair("csm.remote_workload_type", "gcp_compute_engine"),
+          Pair("csm.remote_workload_canonical_service", "canonical_service"),
+          Pair("csm.remote_workload_name", "workload"),
+          Pair("csm.remote_workload_location", "zone"),
+          Pair("csm.remote_workload_project_id", "id")))
+      << PrettyPrintLabels(labels);
+}
+
+TEST(MeshLabelsIterableTest, RemoteGkeTypeMetadata) {
+  std::vector<std::pair<absl::string_view, std::string>> local_labels = {
+      {"csm.workload_canonical_service", "canonical_service"},
+      {"csm.mesh_id", "mesh"}};
+  grpc::internal::MeshLabelsIterable iterable(
+      local_labels, RemoteMetadataSliceFromResource(TestGkeResource()));
+  auto labels = LabelsFromIterable(&iterable);
+  EXPECT_TRUE(iterable.GotRemoteLabels());
+  EXPECT_THAT(
+      labels,
+      ElementsAre(
+          Pair("csm.workload_canonical_service", "canonical_service"),
+          Pair("csm.mesh_id", "mesh"),
+          Pair("csm.remote_workload_type", "gcp_kubernetes_engine"),
+          Pair("csm.remote_workload_canonical_service", "canonical_service"),
+          Pair("csm.remote_workload_name", "workload"),
+          Pair("csm.remote_workload_namespace_name", "namespace"),
+          Pair("csm.remote_workload_cluster_name", "cluster"),
+          Pair("csm.remote_workload_location", "region"),
+          Pair("csm.remote_workload_project_id", "id")))
+      << PrettyPrintLabels(labels);
+}
+
+TEST(MeshLabelsIterableTest, RemoteUnknownTypeMetadata) {
+  std::vector<std::pair<absl::string_view, std::string>> local_labels = {
+      {"csm.workload_canonical_service", "canonical_service"},
+      {"csm.mesh_id", "mesh"}};
+  grpc::internal::MeshLabelsIterable iterable(
+      local_labels, RemoteMetadataSliceFromResource(TestUnknownResource()));
+  auto labels = LabelsFromIterable(&iterable);
+  EXPECT_TRUE(iterable.GotRemoteLabels());
+  EXPECT_THAT(
+      labels,
+      ElementsAre(
+          Pair("csm.workload_canonical_service", "canonical_service"),
+          Pair("csm.mesh_id", "mesh"),
+          Pair("csm.remote_workload_type", "random"),
+          Pair("csm.remote_workload_canonical_service", "canonical_service")))
+      << PrettyPrintLabels(labels);
+}
+
+TEST(MeshLabelsIterableTest, TestResetIteratorPosition) {
+  std::vector<std::pair<absl::string_view, std::string>> local_labels = {
+      {"csm.workload_canonical_service", "canonical_service"},
+      {"csm.mesh_id", "mesh"}};
+  grpc::internal::MeshLabelsIterable iterable(local_labels, grpc_core::Slice());
+  auto labels = LabelsFromIterable(&iterable);
+  auto expected_labels_matcher = ElementsAre(
+      Pair("csm.workload_canonical_service", "canonical_service"),
+      Pair("csm.mesh_id", "mesh"), Pair("csm.remote_workload_type", "unknown"),
+      Pair("csm.remote_workload_canonical_service", "unknown"));
+  EXPECT_THAT(labels, expected_labels_matcher) << PrettyPrintLabels(labels);
+  // Resetting the iterable should return the entire list again.
+  iterable.ResetIteratorPosition();
+  labels = LabelsFromIterable(&iterable);
+  EXPECT_THAT(labels, expected_labels_matcher) << PrettyPrintLabels(labels);
+}
+
 INSTANTIATE_TEST_SUITE_P(
     MetadataExchange, MetadataExchangeTest,
     ::testing::Values(
