@@ -32,6 +32,7 @@
 
 #include "absl/algorithm/container.h"
 #include "absl/flags/flag.h"
+#include "absl/log/check.h"
 #include "absl/strings/str_split.h"
 #include "opentelemetry/exporters/prometheus/exporter_factory.h"
 #include "opentelemetry/exporters/prometheus/exporter_options.h"
@@ -50,7 +51,7 @@
 #include "src/proto/grpc/testing/empty.pb.h"
 #include "src/proto/grpc/testing/messages.pb.h"
 #include "src/proto/grpc/testing/test.grpc.pb.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 #include "test/cpp/interop/rpc_behavior_lb_policy.h"
 #include "test/cpp/interop/xds_stats_watcher.h"
 #include "test/cpp/util/test_config.h"
@@ -217,7 +218,7 @@ class TestClient {
     bool ok = false;
     while (cq_.Next(&got_tag, &ok)) {
       AsyncClientCall* call = static_cast<AsyncClientCall*>(got_tag);
-      GPR_ASSERT(ok);
+      CHECK(ok);
       {
         std::lock_guard<std::mutex> lock(stats_watchers_->mu);
         auto server_initial_metadata = call->context.GetServerInitialMetadata();
@@ -269,7 +270,7 @@ class TestClient {
   static bool RpcStatusCheckSuccess(AsyncClientCall* call) {
     // Determine RPC success based on expected status.
     grpc_status_code code;
-    GPR_ASSERT(grpc_status_code_from_string(
+    CHECK(grpc_status_code_from_string(
         absl::GetFlag(FLAGS_expect_status).c_str(), &code));
     return code ==
            static_cast<grpc_status_code>(call->result.status.error_code());
@@ -341,8 +342,8 @@ class XdsUpdateClientConfigureServiceImpl
     std::vector<RpcConfig> configs;
     int request_payload_size = absl::GetFlag(FLAGS_request_payload_size);
     int response_payload_size = absl::GetFlag(FLAGS_response_payload_size);
-    GPR_ASSERT(request_payload_size >= 0);
-    GPR_ASSERT(response_payload_size >= 0);
+    CHECK_GE(request_payload_size, 0);
+    CHECK_GE(response_payload_size, 0);
     for (const auto& rpc : request->types()) {
       RpcConfig config;
       config.timeout_sec = request->timeout_sec();
@@ -419,7 +420,7 @@ void RunTestLoop(std::chrono::duration<double> duration_per_query,
         } else if (config.type == ClientConfigureRequest::UNARY_CALL) {
           client.AsyncUnaryCall(config);
         } else {
-          GPR_ASSERT(0);
+          CHECK(0);
         }
       }
     }
@@ -447,7 +448,7 @@ grpc::CsmObservability EnableCsmObservability() {
 
 void RunServer(const int port, StatsWatchers* stats_watchers,
                RpcConfigurationsQueue* rpc_configs_queue) {
-  GPR_ASSERT(port != 0);
+  CHECK_NE(port, 0);
   std::ostringstream server_address;
   server_address << "0.0.0.0:" << port;
 
@@ -479,7 +480,7 @@ void BuildRpcConfigsFromFlags(RpcConfigurationsQueue* rpc_configs_queue) {
   for (auto& data : rpc_metadata) {
     std::vector<std::string> metadata =
         absl::StrSplit(data, ':', absl::SkipEmpty());
-    GPR_ASSERT(metadata.size() == 3);
+    CHECK_EQ(metadata.size(), 3u);
     if (metadata[0] == "EmptyCall") {
       metadata_map[ClientConfigureRequest::EMPTY_CALL].push_back(
           {metadata[1], metadata[2]});
@@ -487,7 +488,7 @@ void BuildRpcConfigsFromFlags(RpcConfigurationsQueue* rpc_configs_queue) {
       metadata_map[ClientConfigureRequest::UNARY_CALL].push_back(
           {metadata[1], metadata[2]});
     } else {
-      GPR_ASSERT(0);
+      CHECK(0);
     }
   }
   std::vector<RpcConfig> configs;
@@ -495,8 +496,8 @@ void BuildRpcConfigsFromFlags(RpcConfigurationsQueue* rpc_configs_queue) {
       absl::StrSplit(absl::GetFlag(FLAGS_rpc), ',', absl::SkipEmpty());
   int request_payload_size = absl::GetFlag(FLAGS_request_payload_size);
   int response_payload_size = absl::GetFlag(FLAGS_response_payload_size);
-  GPR_ASSERT(request_payload_size >= 0);
-  GPR_ASSERT(response_payload_size >= 0);
+  CHECK_GE(request_payload_size, 0);
+  CHECK_GE(response_payload_size, 0);
   for (const std::string& rpc_method : rpc_methods) {
     RpcConfig config;
     if (rpc_method == "EmptyCall") {
@@ -504,7 +505,7 @@ void BuildRpcConfigsFromFlags(RpcConfigurationsQueue* rpc_configs_queue) {
     } else if (rpc_method == "UnaryCall") {
       config.type = ClientConfigureRequest::UNARY_CALL;
     } else {
-      GPR_ASSERT(0);
+      CHECK(0);
     }
     auto metadata_iter = metadata_map.find(config.type);
     if (metadata_iter != metadata_map.end()) {
@@ -541,8 +542,8 @@ int main(int argc, char** argv) {
   grpc::testing::InitTest(&argc, &argv, true);
   // Validate the expect_status flag.
   grpc_status_code code;
-  GPR_ASSERT(grpc_status_code_from_string(
-      absl::GetFlag(FLAGS_expect_status).c_str(), &code));
+  CHECK(grpc_status_code_from_string(absl::GetFlag(FLAGS_expect_status).c_str(),
+                                     &code));
   StatsWatchers stats_watchers;
   RpcConfigurationsQueue rpc_config_queue;
 
